# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Andrew Breiner breinera@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record
import gtk


from blogite.helpers import get_builder

from blogite import (
    FeedDialog)

import gettext
from gettext import gettext as _
gettext.textdomain('blogite')


class PreferencesBlogiteDialog(gtk.Dialog):
    __gtype_name__ = "PreferencesBlogiteDialog"
    preferences = {}

    def __new__(cls):
        """Special static method that's automatically called by Python when
        constructing a new instance of this class.

        Returns a fully instantiated PreferencesBlogiteDialog object.
        """
        builder = get_builder('PreferencesBlogiteDialog')
        new_object = builder.get_object("preferences_blogite_dialog")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initalizing should be called after parsing the ui definition
        and creating a PreferencesBlogiteDialog object with it in order to
        finish initializing the start of the new PerferencesBlogiteDialog
        instance.

        Put your initialization code in here and leave __init__ undefined.
        """

        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.builder.connect_signals(self)

        # Set up couchdb and the preference info.
        self.db_pref = CouchDatabase("blogite_preferences", create=True)
        self.pref_type = "https://wiki.ubuntu.com/blogite_preferences.html"
        prefs = self.db_pref.get_records(record_type=self.pref_type, \
                                         create_view=True)

        self.builder.get_object("refresh_spinbutton").set_value(5)
        for records in prefs:
            record = records.value
            try:
                self.builder.get_object( \
                    "refresh_spinbutton").set_value(record["refesh"])
            except:
                pass

        # Set up couchdb and the feed info.
        self.db_feed = CouchDatabase("blogite_feeds", create=True)
        self.feed_type = "https://wiki.ubuntu.com/blogite_feed.html"

        self.treeview = gtk.TreeView()
        self.treeview.columns = [None] * 2
        self.treeview.columns[0] = gtk.TreeViewColumn('Feed URL')
        self.treeview.columns[1] = gtk.TreeViewColumn('Image URL')

        self.liststore = gtk.ListStore(str, str)
        self.modelfilter = self.liststore.filter_new()
        feeds = self.db_feed.get_records(record_type=self.feed_type)
        for records in feeds:
            record = records.value
            try:
                temp = [record["feed_url"], record["image_url"]]
                try:
                    self.liststor.index(temp)
                except:
                    self.liststore.append(temp)
            except:
                pass

        self.treeview.set_model(self.modelfilter)

        for n in range(2):
            self.treeview.append_column(self.treeview.columns[n])
            self.treeview.columns[n].cell = gtk.CellRendererText()
            self.treeview.columns[n].pack_start( \
                self.treeview.columns[n].cell, True)
            self.treeview.columns[n].set_attributes( \
                self.treeview.columns[n].cell, text=n)

        self.treeview.set_search_column(2)
        self.builder.get_object("feed_scrollwindow").add(self.treeview)
        self.show_all()

    def update_feed_list(self):
        self.liststore = gtk.ListStore(str, str)
        self.modelfilter = self.liststore.filter_new()
        feeds = self.db_feed.get_records(record_type=self.feed_type)
        for records in feeds:
            record = records.value
            try:
                temp = [record["feed_url"], record["image_url"]]
                try:
                    self.liststor.index(temp)
                except:
                    self.liststore.append(temp)
            except:
                pass

        self.treeview.set_model(self.modelfilter)
        self.treeview.set_search_column(2)
        self.show_all()

    def add_feed(self, widget, data=None):
        """The user has elected to add a new feed."""
        add_feed = FeedDialog.FeedDialog()
        response = add_feed.run()
        if response == gtk.RESPONSE_OK:
            feed_url, image_url = add_feed.get_urls()
            rec = {"feed_url": feed_url, "image_url": image_url}
            feeds = self.db_feed.get_records(record_type=self.feed_type)
            for records in feeds:
                record = records.value
                if record["feed_url"] == feed_url:
                    key = record["_id"]
                    self.db_feed.update_fields(key, rec)
                    add_feed.destroy()
                    self.update_feed_list()
                    return
            db_record = Record(rec, self.feed_type)
            self.db_feed.put_record(db_record)
            self.update_feed_list()
        add_feed.destroy()

    def edit_feed(self, widget, data=None):
        """The user has elected to edit a feed."""
        try:
            edit_feed = FeedDialog.FeedDialog()
            update_feed = self.get_selected_item()
            edit_feed.set_urls(self.treestore.get_value(self.iter, 0), \
                               self.treestore.get_value(self.iter, 1))
            response = edit_feed.run()
            if response == gtk.RESPONSE_OK:
                feed_url, image_url = edit_feed.get_urls()
                rec = {"feed_url": feed_url, "image_url": image_url}
                feeds = self.db_feed.get_records(record_type=self.feed_type)
                for records in feeds:
                    record = records.value
                    if record["feed_url"] == update_feed:
                        key = record["_id"]
                        self.db_feed.update_fields(key, rec)
                        edit_feed.destroy()
                        self.update_feed_list()
                        return
                db_record = Record(rec, self.feed_type)
                self.db_feed.put_record(db_record)
                self.update_feed_list()
            edit_feed.destroy()
        except:
            pass

    def delete_feed(self, widget, data=None):
        """The user has elected to delete a feed."""
        try:
            feed_url = self.get_selected_item()
            feeds = self.db_feed.get_records(record_type=self.feed_type)
            for records in feeds:
                record = records.value
                if record["feed_url"] == feed_url:
                    key = record["_id"]
                    self.db_feed.delete_record(key)
        except:
            pass
        self.update_feed_list()

    def get_selected_item(self):
        self.treeselection = self.treeview.get_selection()
        (self.model, self.iter) = self.treeselection.get_selected()
        self.treestore, self.paths = \
            self.treeview.get_selection().get_selected_rows()
        return self.treestore.get_value(self.iter, 0)

    def ok(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns gtk.RESONSE_OK from run().
        """
        rec = {"refesh": self.builder.get_object( \
                "refresh_spinbutton").get_value()}
        prefs = self.db_pref.get_records(record_type=self.pref_type)
        for records in prefs:
            record = records.value
            key = record["_id"]
            self.db_pref.update_fields(key, rec)
            return

        db_record = Record(rec, self.pref_type)
        self.db_pref.put_record(db_record)

    def cancel(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """
        # Restore any changes to self._preferences here.
        pass

if __name__ == "__main__":
    dialog = PreferencesBlogiteDialog()
    dialog.show()
    gtk.main()
